import checkTokenIsValid from "@backend/components/checkTokenIsValid"
import createTokenDB from "@backend/components/createTokenDB"
import getPlayerByIdDB from "@backend/components/getPlayerByIdDB"
import getTokenDB from "@backend/components/getTokenDB"
import getTokenFromCookie from "@backend/components/getTokenFromCookie"
import sendError from "@backend/components/sendError"
import sendResponse from "@backend/components/sendResponse"
import { Player, Token } from "@prisma/client"
import type { NextApiRequest, NextApiResponse } from "next"

interface Data {
  token: string
}

export default async function auth(
  req: NextApiRequest,
  res: NextApiResponse<Data>
) {
  const context = { req, res }
  const type = "ACCESS"

  return getTokenFromCookie(context, (refreshToken, newPlayer) => {
    const next = (player: Player, tokenDB: Token, cookie?: string) =>
      createTokenDB(player, type, (newToken, newTokenDB) =>
        sendResponse(context, {
          message: `Access-Token generated: ${newTokenDB.id} with Refreshtoken-Token: ${tokenDB.id}`,
          body: { token: newToken.value },
          type: ["debug", "infoCyan"],
          cookie,
        })
      )
    if (!newPlayer) {
      return checkTokenIsValid(context, refreshToken, (token) =>
        getTokenDB(context, token, (tokenDB) =>
          getPlayerByIdDB(context, tokenDB, (player) => next(player, tokenDB))
        )
      )
    }
    const { player, newToken, newTokenDB } = newPlayer
    return next(player, newTokenDB, newToken.value)
  }).catch((err) => sendError(context, err))
}
